﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PuzzlePlayer_Namespace
{
    internal class PuzzleForm : Form
    {
        private readonly Button solvebutton;
        private readonly Button hintbutton;
        private readonly Button generatebutton;
        private readonly Button restartbutton;
        private readonly Button UPDATEBUTTON;
        private readonly Label titlebox;
        private readonly Label informationbox;
        private Rectangle boardspace;
        private readonly MenuStrip menuStrip;
        private readonly ToolStripMenuItem menuSettings;
        private readonly BufferedGraphics bufferedGraphics;
        private Board board;
        public Board Board //updating the Board member will immediately call board.Draw method so that the board is updated visually
        {
            get { return board; }
            set
            {
                board = value;
                this.Invalidate();
            }
        }
        public readonly string puzzleType;

        public PuzzleForm(Board b, Size s = default) //takes Board and Size parameter and sets up the PuzzleForm window.
        {
            puzzleType = b.GetType().Name;
            if (s == default) s = new Size(700, 420);
            this.Size = this.MinimumSize = s;
            this.WindowState = FormWindowState.Maximized;
            this.BackColor = SettingForm.primaryColor;
            this.Text = "PuzzlePlayer";
            bufferedGraphics = BufferedGraphicsManager.Current.Allocate(this.CreateGraphics(), this.DisplayRectangle);
            bufferedGraphics.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            generatebutton = new Button();
            hintbutton = new Button();
            solvebutton = new Button();
            restartbutton = new Button();
            UPDATEBUTTON = new Button();
            titlebox = new Label();
            informationbox = new Label();
            boardspace = new Rectangle(220, 55, 400, 400);
            this.Paint += (object o, PaintEventArgs pea) =>
            {
                board.Draw(bufferedGraphics.Graphics, boardspace);
                //bufferedGraphics.Graphics.FillRectangle(Brushes.LightCoral, boardspace.X, boardspace.Y, this.Width - 350, this.Height - 125);
                //bufferedGraphics.Graphics.FillRectangle(Brushes.DarkRed, boardspace);
                bufferedGraphics.Render();
            };
            this.Resize += (object o, EventArgs ea) => UpdateUI();
            this.Move += (object o, EventArgs ea) => this.Focus();
            this.PreviewKeyDown += (object o, PreviewKeyDownEventArgs e) => e.IsInputKey = true;
            this.KeyDown += (object o, KeyEventArgs kea) => Input(kea.KeyCode);
            this.MouseClick += (object o, MouseEventArgs mea) =>
            {
                if (mea.Button == MouseButtons.Left)
                {
                    Input(Keys.LButton);
                    return;
                }
                if (mea.Button == MouseButtons.Right) Input(Keys.RButton);
            };

            Board = b;
            CreateUI();
        }

        private void CreateUI() //sets up ui elements
        {
            MenuStrip menuStrip = new MenuStrip
            {
                BackColor = SettingForm.secondaryColor,
                ForeColor = SettingForm.secondaryColor,
                Name = "Main menu",
                Text = "Main Menu",
                Dock = DockStyle.Top,
                Font = SettingForm.mainFont,
            };
            ToolStripMenuItem menuSettings = new ToolStripMenuItem
            {
                BackColor = Color.FromArgb(54, 57, 62),
                ForeColor = Color.Black,
                Text = "Settings",
                TextAlign = ContentAlignment.BottomRight,
                Font = SettingForm.mainFont,
            };
            menuStrip.Items.Add(menuSettings);
            this.Controls.Add(menuStrip);

            menuSettings.Click += (object o, EventArgs e) =>
            {
                this.Hide();
                SettingForm settingForm = new SettingForm();

                settingForm.FormClosed += (object s, FormClosedEventArgs args) =>
                {
                    this.BackColor = SettingForm.primaryColor;
                    this.ForeColor = SettingForm.tertiaryColor;

                    foreach (Control control in this.Controls)
                    {
                        SettingForm.UpdateSettings(control);
                    }

                    bufferedGraphics.Graphics.Clear(this.BackColor);

                    this.Show();
                };

                settingForm.Show();
            };

            void CreateButton(Button b)
            {
                Controls.Add(b);
                b.Size = new Size(80, 50);
                b.FlatAppearance.BorderColor = b.BackColor = Color.Gainsboro;
                b.Text = "DEFAULT0";
                b.Font = new Font("Verdana", 11, FontStyle.Bold | FontStyle.Italic);
                b.FlatStyle = FlatStyle.Flat;
                b.GotFocus += (object o, EventArgs ea) => this.Focus();
            }
            CreateButton(generatebutton);
            generatebutton.Text = "New Game";
            generatebutton.MouseClick += (object sender, MouseEventArgs mea) => 
            {
                Board.Generate();
                this.Invalidate();
            };
            CreateButton(hintbutton);
            hintbutton.Text = "Hint";
            hintbutton.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                MessageBox.Show("Hint: geef op");
                this.Invalidate();
            };
            CreateButton(solvebutton);
            solvebutton.Text = "Solve";
            solvebutton.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                Board.Solve(false);
                this.Invalidate();
            };
            CreateButton(restartbutton);
            restartbutton.Text = "Restart";
            restartbutton.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                Board.Restart();
                this.Invalidate();
            };

            //CreateButton(UPDATEBUTTON);
            UPDATEBUTTON.Text = "Update";
            UPDATEBUTTON.Font = new Font("Verdana", 11, FontStyle.Bold);
            UPDATEBUTTON.FlatAppearance.BorderColor = UPDATEBUTTON.BackColor = Color.Pink;
            UPDATEBUTTON.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                this.Invalidate();
            };

            Controls.Add(titlebox);
            titlebox.Location = new Point(20, 40);
            titlebox.Size = new Size(180, 50);
            titlebox.BackColor = this.BackColor;
            titlebox.ForeColor = Color.White;
            titlebox.Text = puzzleType;
            titlebox.Font = new Font("Verdana", 24, FontStyle.Bold);

            Controls.Add(informationbox);
            informationbox.Location = new Point(20, 90);
            informationbox.Size = new Size(180, 300);
            informationbox.BackColor = this.BackColor;
            informationbox.ForeColor = Color.White;
            informationbox.Text = board.description;
            informationbox.Font = new Font("Verdana", 10);

            this.BackColor = SettingForm.primaryColor;
            this.ForeColor = SettingForm.tertiaryColor;

            UpdateUI();
        }
        private void UpdateUI() //resizes the boardspace rectangle and updates the rest of the ui around the new boardspace size
        {
            bufferedGraphics.Graphics.Clear(this.BackColor);
            boardspace.Size = FitBoard(new Size(Board.boardState.GetLength(0), Board.boardState.GetLength(1)), new Size(this.Width - 350, this.Height - 100), Board.drawFactor);
            generatebutton.Location = new Point(boardspace.Right + 30, 30);
            hintbutton.Location = new Point(boardspace.Right + 30, 110);
            solvebutton.Location = new Point(boardspace.Right + 30, 190);
            restartbutton.Location = new Point(boardspace.Right + 30, 270);
            UPDATEBUTTON.Location = new Point(boardspace.Right + 30, 350);
            this.Invalidate();
        }
        public static Size FitBoard(Size gamesize, Size maxboardsize, int drawfactor) //returns the largest rectangle smaller than MaxBoardSize that fits the given GameSize well
        {
            int TileLength = Math.Min(maxboardsize.Width / gamesize.Width, maxboardsize.Height / gamesize.Height) / drawfactor * drawfactor;
            return new Size(Math.Max(6,gamesize.Width*TileLength), Math.Max(6,gamesize.Height*TileLength)); //dw dw
            //after each of the divisions in this method, the value should be rounded down, 
            //as the end value cannot exceed the size given by maxboardsize.
            //however, C# rounds divisions of Int values down by default, so this does not have to be explicitly done.
        }
        private Point GetTile(Size gamesize, Rectangle r, Point p) //returns board coordinate at mouse position P of a Gamesize board drawn in R
        {
            Point res = this.PointToClient(p) - (Size)r.Location;
            res.X /= (r.Width / gamesize.Width);
            res.Y /= (r.Height / gamesize.Height);
            return res;
        }
        public void Input(Keys k) //checks if a command binded to the keyboard key and runs it, affects tile that is hovered on if applicable
        {
            switch (k)
            {
                case Keys.N:
                    Board.Generate();
                    this.Invalidate();
                    return;
                case Keys.H:
                    MessageBox.Show("Hint: geef op");
                    this.Invalidate();
                    return;
                case Keys.Space:
                    Board.Solve(false);
                    this.Invalidate();
                    return;
                case Keys.Up:
                case Keys.Left:
                case Keys.Right:
                case Keys.Down:
                case Keys.W:
                case Keys.A:
                case Keys.S:
                case Keys.D:
                    if (Board.TileInput(null, k)) Win();
                    this.Invalidate();
                    return;
            }
            Point tile = GetTile(new Size(Board.boardState.GetLength(0), Board.boardState.GetLength(1)), boardspace, Control.MousePosition);
            if (!(tile.X >= 0 && tile.X < Board.boardState.GetLength(0) && tile.Y >= 0 && tile.Y < Board.boardState.GetLength(1))) return; //if tile is out of bounds
            if (!(Board.lastGeneratedBoard[tile.X, tile.Y] == Board.emptySpace)) return; //if tile is a given
            switch (k)
            {
                case Keys.Back:
                    Board.boardState[tile.X, tile.Y] = Board.emptySpace;
                    this.Invalidate();
                    return;
                case Keys.LButton:
                case Keys.RButton:
                    if (Board.TileClick(tile, (int)k - 1)) Win();
                    this.Invalidate();
                    return;
                case Keys.OemOpenBrackets:
                case Keys.OemCloseBrackets:
                    if (Board.TileClick(tile, ((int)k - 219)/2)) Win();
                    this.Invalidate();
                    return;
                default:
                    if (Board.TileInput(tile, k)) Win();
                    this.Invalidate();
                    return;
            }
        }

        public void Win()
        {
            this.Invalidate();
            MessageBox.Show("oi oi oi");
        }
    }
}
